/*
 * Copyright (c) 2023-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <config.h>
#include "dpdk.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <rte_devargs.h>
#include <rte_flow.h>
#include <rte_pci.h>
#include <rte_version.h>

#include "openvswitch/vlog.h"
#include "smap.h"

VLOG_DEFINE_THIS_MODULE(dpdk_ext);

const char *
dpdk_get_version(void)
{
    return rte_version();
}

bool
dpdk_parse_devargs_pci(const char *devargs, struct rte_pci_addr *rte_pci)
{
    struct rte_devargs da;
    bool rv = true;

    if (rte_devargs_parse(&da, devargs)) {
        return false; /* Error has already been printed by the RTE function. */
    }

    if (rte_pci_addr_parse(da.name, rte_pci)) {
        rv = false;
        goto out;
    }

out:
    rte_devargs_reset(&da);
    return rv;
}

static void
dump_mempool_stats(struct rte_mempool *mp, void *arg)
{
    FILE *stream = arg;

    rte_mempool_dump(stream, mp);
}

void
malloc_dump_mempool_stats_wrapper(FILE *stream)
{
    rte_mempool_walk(dump_mempool_stats, stream);
}

void
malloc_dump_memzone_stats_wrapper(FILE *stream)
{
    rte_memzone_dump(stream);
}

void
dpdk_init_max_memzones(const struct smap *ovs_other_config)
{
    uint32_t max_memzones;

    max_memzones = smap_get_uint(ovs_other_config, "dpdk-max-memzones",
                                 UINT32_MAX);
    if (max_memzones != UINT32_MAX) {
        int rv;

        rv = rte_memzone_max_set(max_memzones);
        if (rv) {
            VLOG_WARN("Failed to set max memzones to %"PRIu32, max_memzones);
        } else {
            VLOG_INFO("Setting max memzones to %"PRIu32, max_memzones);
        }
    }
}
